@@ -2,20 +2,20 @@
 #'
 #' This function is made to be used in \link{normalize_intensity}. It corrects intensity with a
 #' range correction according to the formula (see references):
-#' \deqn{I_{norm} = I_{obs} (\frac{R}{Rs})^f)}{Inorm = Iobs * (R/Rs)^f}
+#' \deqn{I_{norm} = I_{obs} \left(\frac{R}{Rs}\right)^f}{Inorm = Iobs * (R/Rs)^f}
 #' To achieve the range correction the position of the sensor must be known at different discrete times.
 #' Using the 'gpstime' of each point, the position of the sensor is interpolated from the reference
 #' and a range correction is applied.
 #'
-#' @param sensor \code{SpatialPointsDataDrame} object containing the coordinates of
+#' @param sensor `SpatialPointsDataDrame` or `sf` object containing the coordinates of
 #' the sensor at different timepoints t. The time and elevation are stored as attributes
-#' (default names are 'gpstime' and 'Z'). It can be computed with \link{track_sensor}.
+#' (default names are 'gpstime' and 'Z'). Z can also come from the geometry if the input records XYZ
+#' coordinates. It can be computed with \link{track_sensor}.
 #' @param Rs numeric. Range of reference.
 #' @param f numeric. Exponent. Usually between 2 and 3 in vegetation contexts.
 #' @param gpstime,elevation character. The name of the attributes that store the gpstime of the
-#' position and the elevation of the sensor respectively. If \code{elevation = NULL} the Z coordinates
-#' are searched in the third column of the coordinates matrix of the SpatialPointsDataFrame. This is
-#' useful if read from a format that supports 3 coordinates points.
+#' position and the elevation of the sensor respectively. If the input contains 3 coordinates points,
+#' `elevation` is not considered.
 #' @param las an object of class LAS. \code{get_range()} is a regular function documented here for
 #' convenience.
 #' @export
@@ -45,7 +45,6 @@ range_correction = function(sensor, Rs, f = 2.3, gpstime = "gpstime", elevation
   assert_is_a_number(f)
   assert_all_are_positive(Rs)
   assert_all_are_positive(f)
-  assert_is_all_of(sensor, "SpatialPointsDataFrame")
 
   Rs <- lazyeval::uq(Rs)
   f <- lazyeval::uq(f)
@@ -75,54 +74,62 @@ get_range = function(las, sensor, gpstime = "gpstime", elevation = "Z")
 
 prepare_sensor = function(sensor, gpstime, elevation, las)
 {
-  stopifnot(is(sensor, "SpatialPointsDataFrame"))
+  if (is(sensor, "SpatialPointsDataFrame"))
+    sensor <- sf::st_as_sf(sensor)
 
-  if (!"gpstime"   %in% names(las@data))
+  if (!is(sensor, "sf"))
+    stop("'ttops' is not a SpatialPointsDataFrame or sf")
+
+  geom <- sf::st_geometry(sensor)
+  if (!is(geom, "sfc_POINT"))
+    stop("Only point geometry types are supported", call. = FALSE)
+
+  if (!"gpstime" %in% names(las))
     stop("No 'gpstime' attribute found in las",   call. = FALSE)
 
-  if (!"Intensity" %in% names(las@data))
+  if (!"Intensity" %in% names(las))
     stop("No 'Intensity' attribute found", call. = FALSE)
 
-  # Relaxed in #395
-  #if (is.null(raster::intersect(raster::extent(las), raster::extent(sensor))))
-  #  stop("Point-cloud and sensor positions do not overlap.", call. = FALSE)
-
-  if (!gpstime %in% names(sensor@data))
+  if (!gpstime %in% names(sensor))
     stop(glue::glue("No '{gpstime}' attribute found in sensor."), call. = FALSE)
 
-  if (ncol(sensor@coords) == 3 && !is.null(elevation))
+  if (is(sf::st_geometry(sensor)[[1]], "XYZ"))
   {
     elevation = NULL
-    message("3 coordinates detected in the sensor positions, parameter 'elevation' was not considered.")
+    message("3 coordinates detected in the sensor positions, parameter 'elevation' is not considered.")
   }
 
-  fl <- sensor@coords
+  fl <- sf::st_coordinates(sensor)
   fl <- data.table::data.table(fl)
 
-  trange.las <- range(las@data[["gpstime"]])
-  trange.sensor <- range(sensor@data[[gpstime]])
+  trange.las <- range(las[["gpstime"]])
+  trange.sensor <- range(sensor[[gpstime]])
 
   if ((trange.las[1] < trange.sensor[1] - 2) || (trange.las[2] > trange.sensor[2] + 2))
     stop("'gpstime range from the sensor does not contain gpstime range from the point-cloud", call. = FALSE)
 
-  if (!is.null(elevation)) {
-    if (!elevation %in% names(sensor@data))
+  if (!is.null(elevation))
+  {
+    if (!elevation %in% names(sensor))
       stop(glue::glue("No '{elevation}' attribute found in sensor."), call. = FALSE)
 
-    fl[["Z"]] <- sensor@data[[elevation]]
-  } else {
-    if (ncol(sensor@coords) != 3)
+    fl[["Z"]] <- sensor[[elevation]]
+  }
+  else
+  {
+    if (ncol(fl) != 3)
       stop("There are only two dimensions in the coordinates of the sensor.", call. = FALSE)
   }
 
-  fl[["gpstime"]] <- sensor@data[[gpstime]]
+  fl[["gpstime"]] <- sensor[[gpstime]]
 
   fl <- data.table::data.table(fl)
   data.table::setorder(fl, gpstime)
   data.table::setnames(fl, c("X", "Y", "Z", "gpstime"))
   dup <- duplicated(fl, by = "gpstime")
 
-  if (any(dup)) {
+  if (any(dup))
+  {
     warning("Duplicated gpstime found. Duplicated sensor positions were removed.", call. = FALSE)
     fl <- fl[!dup]
   }
