@@ -1,97 +1,28 @@
-# ===============================================================================
-#
-# PROGRAMMERS:
-#
-# jean-romain.roussel.1@ulaval.ca  -  https://github.com/Jean-Romain/lidR
-#
-# COPYRIGHT:
-#
-# Copyright 2017-2018 Jean-Romain Roussel
-#
-# This file is part of lidR R package.
-#
-# lidR is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-#
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>
-#
-# ===============================================================================
-
 # ======== GENERIC =========
 
 #' Clip points in regions of interest
 #'
-#' Clip points within a given region of interest (ROI) from a point cloud (\code{LAS} object) or a catalog
-#' (\code{LAScatalog} object). With a \code{LAS} object, the user first reads and loads a point cloud
-#' into memory and then can clip it to get a subset within a region of interest. With a \code{LAScatalog}
-#' object, the user can extract any arbitrary ROI for a set of \code{las/laz} files, loading only the
-#' points of interest. This is faster, easier and much more memory-efficient for extracting ROIs.
+#' Clip points within a given region of interest (ROI) from a point cloud (LAS object) or a collection
+#' of files (LAScatalog object).
 #'
 #' @template param-las
-#' @param geometry a geometric object. Many types are supported, see section 'supported geometries'.
-#' @param xleft numeric. left x coordinates of rectangles.
-#' @param ybottom	numeric. bottom y coordinates of rectangles.
-#' @param xright numeric. right x coordinates of rectangles.
-#' @param ytop numeric. top y coordinates of rectangles.
-#' @param xpoly numeric. x coordinates of a polygon.
-#' @param ypoly numeric. y coordinates of a polygon.
-#' @param xcenter numeric. x coordinates of disc centers.
-#' @param ycenter numeric. y coordinates of disc centers.
+#' @param geometry a geometric object. spatial points, spatial polygons in sp or sf/sfc format, Extent,
+#' bbox, 2x2 matrix
+#' @param xleft,ybottom,xright,ytop numeric. coordinates of one or several rectangles.
+#' @param xpoly,ypoly numeric. x coordinates of a polygon.
+#' @param xcenter,ycenter numeric. x coordinates of on or several disc centres.
 #' @param radius numeric. disc radius or radii.
-#' @param ... in \code{clip_roi}: optional supplementary options (see supported geometries). Unused in
+#' @param ... in `clip_roi`: optional supplementary options (see supported geometries). Unused in
 #' other functions
 #'
-#' @section Supported geometries:
-#' \itemize{
-#'  \item \href{https://en.wikipedia.org/wiki/Well-known_text}{WKT string}: describing a POINT, a POLYGON or
-#'  a MULTIPOLYGON. If points, a parameter 'radius' must be passed in \code{...}
-#'  \item \link[sp:Polygon-class]{Polygon}, \link[sp:SpatialPolygons-class]{SpatialPolygons},
-#'  \link[sp:SpatialPolygonsDataFrame-class]{SpatialPolygonsDataFrame},  \link[sp:SpatialPoints-class]{SpatialPoints}
-#'  or \link[sp:SpatialPointsDataFrame-class]{SpatialPointsDataFrame}
-#'  in that case a parameter 'radius' must be passed in \code{...}
-#'  \item \link[sf:sf]{SimpleFeature} from sf that consistently contains \code{POINT} or \code{POLYGON/MULTIPOLYGON}.
-#'  In case of \code{POINT} a parameter 'radius' must be passed in \code{...}
-#'  \item \link[raster:Extent-class]{Extent} from package \code{raster}
-#'  \item \link[sf:st_bbox]{bbox} from package \code{sf}
-#'  \item \link[base:matrix]{matrix} 2 x 2 describing a bounding box following this order:
-#'  \preformatted{
-#'   min     max
-#' x 684816  684943
-#' y 5017823 5017957}
-#'  }
+#' @section Non-supported LAScatalog options:
+#' The option `chunk size`, `buffer`, `chunk alignment` and `select` are not supported by `clip_*`
+#' because they are meaningless in this context.
 #'
-#' @template LAScatalog
-#'
-#' @section Supported processing options:
-#' Supported processing options for a \code{LAScatalog} (in bold). For more details see the
-#' \link[=LAScatalog-class]{LAScatalog engine documentation}:
-#' \itemize{
-#' \item chunk_size: Does not make sense here.
-#' \item buffer: Not supported yet.
-#' \item alignment: Does not makes sense here.
-#' \item \strong{progress}: Displays a progress estimation.
-#' \item \strong{output_files}: If 'output_files' is set in the catalog, the ROIs will not be returned in R.
-#' They will be written immediately in files. See \link{LAScatalog-class} and examples. The allowed templates in
-#' \code{clip_*} are \code{{XLEFT}, {XRIGHT}, {YBOTTOM}, {YTOP}, {ID}, {XCENTER},
-#' {YCENTER}}. In addition \code{clip_roi} supports any names from the table of attributes of a spatial object given as
-#' input such as \code{{PLOTID}}, \code{{YEAR}}, \code{{SPECIES}}, for examples, if these attributes exist. If empty everything
-#' is returned into R.
-#' \item \strong{laz_compression}: write \code{las} or \code{laz} files
-#' \item select: The function will write files equivalent to the originals. This option is not respected.
-#' \item \strong{filter}: Read only the points of interest.
-#' }
-#'
-#' @return If the input is a \code{LAS} object: an object of class \code{LAS}, or a \code{list} of \code{LAS} objects if the query implies several regions of interest will be returned.\cr\cr
-#' If the input is a \code{LAScatalog} object: an object of class \code{LAS}, or a \code{list} of \code{LAS}
-#' objects if the query implies several regions of interest will be returned, or a \code{LAScatalog} if the
+#' @return If the input is a LAS object: an object of class LAS, or a `list` of LAS objects if the
+#' query implies several regions of interest.\cr\cr
+#' If the input is a LAScatalog object: an object of class LAS, or a `list` of LAS
+#' objects if the query implies several regions of interest, or a LAScatalog if the
 #' queries are immediately written into files without loading anything in R.
 #'
 #' @examples
@@ -130,6 +61,7 @@
 #' plot(tr1, axis = TRUE, clear_artifacts = FALSE)
 #' }
 #' @name clip
+#' @md
 #' @export
 clip_roi = function(las, geometry, ...)
 {
@@ -137,16 +69,13 @@ clip_roi = function(las, geometry, ...)
     geometry <- sf::st_as_sfc(geometry, crs = sf::st_crs(las))
 
   if (is(geometry, "Polygon"))
-    geometry <- sf::st_geometry(sf::st_polygon(list(geometry@coords)))
+    geometry <- sf::st_sfc(sf::st_polygon(list(geometry@coords)), crs = st_crs(las))
 
-  if (is(geometry, "SpatialPolygons") | is(geometry, "SpatialPolygonsDataFrame"))
-    geometry <- sf::st_as_sf(geometry)
-
-  if (is(geometry, "SpatialPoints") | is(geometry, "SpatialPointsDataFrame"))
+  if (inherits(geometry, "Spatial"))
     geometry <- sf::st_as_sf(geometry)
 
   if (is(geometry, "sfg"))
-    geometry <- sf::st_geometry(geometry)
+    geometry <- sf::st_sfc(geometry, crs = st_crs(las))
 
   if (is(geometry, "sf") | is(geometry, "sfc"))
   {
@@ -241,7 +170,7 @@ clip_rectangle.LAS = function(las, xleft, ybottom, xright, ytop, ...)
 #' @export
 clip_rectangle.LAScatalog = function(las, xleft, ybottom, xright, ytop, ...)
 {
-  bboxes  <- mapply(raster::extent, xleft, xright, ybottom, ytop)
+  bboxes  <- st_make_bboxes(xleft, xright, ybottom, ytop)
   output  <- catalog_extract(las, bboxes, LIDRRECTANGLE)
 
   if (length(output) == 1)
@@ -260,7 +189,7 @@ clip_polygon = function(las, xpoly, ypoly, ...)
   assert_is_numeric(ypoly)
   assert_are_same_length(xpoly, ypoly)
 
-  poly <- sp::Polygon(cbind(xpoly, ypoly))
+  poly <- sf::st_polygon(list(cbind(xpoly, ypoly)))
   return(clip_roi(las, poly))
 }
 
@@ -313,7 +242,8 @@ clip_circle.LAScatalog = function(las, xcenter, ycenter, radius, ...)
   xmax   <- xcenter + radius
   ymin   <- ycenter - radius
   ymax   <- ycenter + radius
-  bboxes <- mapply(raster::extent, xmin, xmax, ymin, ymax)
+
+  bboxes <- st_make_bboxes(xmin, xmax, ymin, ymax)
   output <- catalog_extract(las, bboxes, LIDRCIRCLE, ...)
 
   if (length(output) == 1)
@@ -350,9 +280,9 @@ clip_transect = function(las, p1, p2, width, xz = FALSE, ...)
     plot <- isTRUE(p$plot)
     if (plot)
     {
-      dsm <- grid_canopy(las, 2, p2r())
+      dsm <- rasterize_canopy(las, 2, p2r())
       plot(las@header)
-      raster::plot(dsm, col = height.colors(50))
+      plot(dsm, col = height.colors(15))
     }
     tr <- graphics::locator(2L, type = "o")
     p1 <- c(tr$x[1L], tr$y[1L])
@@ -360,7 +290,8 @@ clip_transect = function(las, p1, p2, width, xz = FALSE, ...)
     # nocov end
   }
 
-  if (is(las, "LAScatalog")) {
+  if (is(las, "LAScatalog"))
+  {
     if (xz && opt_output_files(las) != "")
       stop("Reorientation is not available yet with a LAScatalog", call. = FALSE)
   }
@@ -378,10 +309,10 @@ clip_transect = function(las, p1, p2, width, xz = FALSE, ...)
 
   if (!xz) { return(las) }
 
-  zero <- sp::bbox(las)[,1]
+  bbox <- st_bbox(las)
   coords <- as.matrix(coordinates(las))
-  coords[,1] <- coords[,1] - zero[1]
-  coords[,2] <- coords[,2] - zero[2]
+  coords[,1] <- coords[,1] - bbox$xmin
+  coords[,2] <- coords[,2] - bbox$ymin
   coords <- coords %*% rot
   X <- coords[,1]
   Y <- coords[,2]
@@ -391,9 +322,9 @@ clip_transect = function(las, p1, p2, width, xz = FALSE, ...)
   las@data[["Y"]] <- Y
   las@header@PHB[["X offset"]] <- 0
   las@header@PHB[["Y offset"]] <- 0
-  las <- lasupdateheader(las)
+  las <- las_update(las)
   data.table::setattr(las, "rotation", rot)
-  data.table::setattr(las, "offset", zero)
+  data.table::setattr(las, "offset", c(bbox$xmin, bbox$ymin))
   return(las)
 }
 
@@ -406,13 +337,15 @@ clip_sf = function(las, sf)
 
 clip_sf.LAS = function(las, sf)
 {
-  wkt <- sf::st_as_text(sf::st_geometry(sf), digits = 10)
-
-  output = vector(mode = "list", length(wkt))
-  for (i in 1:length(wkt))
+  sfc <- sf::st_geometry(sf)
+  idx <- point_in_polygons(las, sfc)
+  output <- vector(mode = "list", length(sfc))
+  uid <- unique(idx)
+  uid <- uid[!is.na(uid)]
+  for (i in seq_along(uid))
   {
-    roi = filter_poi(las, C_in_polygon(las, wkt[i], getThread()))
-    if (is.empty(roi)) warning(glue::glue("No point found for within {wkt[i]}."))
+    roi = filter_poi(las, idx == uid[i])
+    if (is.empty(roi)) warning(glue::glue("No point found for within {sf::st_as_text(sfc[i])}."), call. = FALSE)
     output[[i]] = roi
   }
 
@@ -430,9 +363,8 @@ clip_sf.LAScatalog = function(las, sf)
 
   # We need the bounding box of each geometry to be able to leverage automatically spatial
   # indexing of LAS files with LAX files
-  bboxes <- lapply(sf::st_geometry(sf), function(x) { raster::extent(sf::st_bbox(x)) })
-
-  output = catalog_extract(las, bboxes, LIDRRECTANGLE, sf = sf)
+  bboxes <- lapply(sf::st_geometry(sf), sf::st_bbox)
+  output <- catalog_extract(las, bboxes, LIDRRECTANGLE, sf = sf)
 
   if (length(output) == 1)
     return(output[[1]])
@@ -443,7 +375,7 @@ clip_sf.LAScatalog = function(las, sf)
 # ============= GENERIC QUERY  =============
 
 #' @param ctg LAScatalog
-#' @param bboxes a list of raster::extent
+#' @param bboxes a list of bbox
 #' @param shape shape of the query can be a rectangle or a disc
 #' @param sf an object of class sf that is used for extracting polygons carrying WKT strings
 #' @param data a data.frame carrying some attributes used to create fill the {TEMPLATE}
@@ -454,7 +386,7 @@ catalog_extract = function(ctg, bboxes, shape = LIDRRECTANGLE, sf = NULL, data =
 
   if (opt_progress(ctg)) plot.LAScatalog(ctg, mapview = FALSE) # nocov
 
-  # Define a function to be passed in cluster_apply
+  # Define a function to be passed in engine_apply
   extract_query = function(cluster)
   {
     if (cluster@files[1] == "")
@@ -476,7 +408,7 @@ catalog_extract = function(ctg, bboxes, shape = LIDRRECTANGLE, sf = NULL, data =
 
   # Find the ROIs in the catalog and return LASclusters. If a ROI fall outside the catalog
   # its associated LAScluster is NULL and must receive a special treatment in following code
-  clusters <- catalog_index(ctg, bboxes, shape, 0, outside_catalog_is_null = FALSE)
+  clusters <- engine_index(ctg, bboxes, shape, 0, outside_catalog_is_null = FALSE)
 
   # Add some information in the clusters to correctly extract polygons and to write correct file names
   for (i in 1:length(clusters))
@@ -545,7 +477,7 @@ catalog_extract = function(ctg, bboxes, shape = LIDRRECTANGLE, sf = NULL, data =
   }
 
   # Process the cluster using LAScatalog internal engine
-  output <- cluster_apply(clusters, extract_query, ctg@processing_options, ctg@output_options)
+  output <- engine_apply(clusters, extract_query, ctg@processing_options, ctg@output_options)
 
   # output should contain nothing because everything has been streamed into files
   if (opt_output_files(ctg) != "")
@@ -573,17 +505,8 @@ catalog_extract = function(ctg, bboxes, shape = LIDRRECTANGLE, sf = NULL, data =
     else
     {
       # Empty LAScatalog
-      LASfile <- system.file("extdata", "Megaplot.laz", package="lidR")
-      new_ctg <- readLAScatalog(LASfile)
-      data <- new_ctg@data
-      data <- data[0,]
-      poly <- sp::SpatialPolygons(list())
-      poly <- sp::SpatialPolygonsDataFrame(poly, data)
-      new_ctg@data <- data
-      new_ctg@polygons <- poly@polygons
-      new_ctg@plotOrder <- poly@plotOrder
-      new_ctg@bbox <- matrix(0,2,2)
-      new_ctg@proj4string <- ctg@proj4string
+      new_ctg <- new("LAScatalog")
+      st_crs(new_ctg) <- st_crs(ctg)
     }
 
     return(list(new_ctg))
@@ -596,7 +519,7 @@ catalog_extract = function(ctg, bboxes, shape = LIDRRECTANGLE, sf = NULL, data =
       if (!is.null(output[[i]]))
       {
         # Transfer the CRS of the catalog.
-        output[[i]]@proj4string <- ctg@proj4string
+        st_crs(output[[i]]) <- st_crs(ctg)
       }
       else
       {
