@@ -26,10 +26,9 @@ class LAS
     //void apply_filter();
     //IntegerVector index_filter();
 
-    void filter_in_polygon(std::string wkt);
     void filter_local_maxima(NumericVector ws, double min_height, bool circular);
     void filter_local_maxima(NumericVector ws);
-    void filter_with_grid(S4 layout, bool max);
+    void filter_with_grid(List layout, bool max);
     void filter_shape(int method, NumericVector th, int k);
     void filter_progressive_morphology(NumericVector ws, NumericVector th);
     void filter_isolated_voxel(double ws, unsigned int isolated);
@@ -40,9 +39,9 @@ class LAS
 
     double range(NumericVector&, NumericVector& , NumericVector&, NumericVector&,  int, double);
 
-    NumericVector rasterize(S4 layout, double subcircle, int method);
+    NumericVector rasterize(List layout, double subcircle, int method);
     NumericVector compute_range(DataFrame flightlines);
-
+    IntegerVector find_polygon_ids(CharacterVector wkts);
     IntegerVector segment_snags(NumericVector neigh_radii, double low_int_thrsh, double uppr_int_thrsh, int pt_den_req, NumericMatrix BBPRthrsh_mat);
     IntegerVector segment_trees(double dt1, double dt2, double Zu, double R, double th_tree, double radius);
     List point_metrics(unsigned int k, double r, DataFrame data, int nalloc, SEXP call, SEXP env);
@@ -54,6 +53,8 @@ class LAS
     static bool coplanar (arma::vec& latent, arma::mat& coeff, NumericVector& th) { return latent[1] > th[0]*latent[2] && th[1]*latent[1] > latent[0]; }
     static bool hcoplanar(arma::vec& latent, arma::mat& coeff, NumericVector& th) { return latent[1] > th[0]*latent[2] && th[1]*latent[1] > latent[0] && std::abs(coeff(2,2)) > th[2]; }
     static bool colinear (arma::vec& latent, arma::mat& coeff, NumericVector& th) { return th[0]*latent[2] < latent[0] && th[0]*latent[1] < latent[0]; }
+    static bool hcolinear(arma::vec& latent, arma::mat& coeff, NumericVector& th) { return th[0]*latent[2] < latent[0] && th[0]*latent[1] < latent[0] && std::abs(coeff(2,0)) < th[1]; }
+    static bool vcolinear(arma::vec& latent, arma::mat& coeff, NumericVector& th) { return th[0]*latent[2] < latent[0] && th[0]*latent[1] < latent[0] && std::abs(coeff(2,0)) > th[1]; }
     static double rmax(double x, double y) { if (NumericVector::is_na(x)) return y; return (x > y) ? x : y; }
     static double rmin(double x, double y) { if (NumericVector::is_na(x)) return y; return (x < y) ? x : y; }
     static double rcount(double x, double y) { if (NumericVector::is_na(x)) return 1; return x+1;}
